@@ -36,7 +36,7 @@
 
 static GObjectClass *parent_class;
 
-static void
+void
 gconf_editor_window_popup_error_dialog (GtkWindow   *parent,
 					const gchar *message,
 					GError      *error)
@@ -76,8 +76,8 @@ gconf_editor_window_go_to (GConfEditorWindow *window,
 
 	g_return_if_fail (location != NULL);
 
-	if (!gconf_client_get (gconf_client_get_default (), location, NULL) && 
-	    !gconf_client_dir_exists (gconf_client_get_default (), location, NULL))
+	if (!gconf_client_get (window->client, location, NULL) && 
+	    !gconf_client_dir_exists (window->client, location, NULL))
 		return;
 
 	child_path = gconf_tree_model_get_tree_path_from_gconf_path (
@@ -187,7 +187,25 @@ gconf_editor_window_new_window (GtkAction *action, GtkWidget *callback_data)
 {
 	GtkWidget *new_window;
 
-	new_window = gconf_editor_application_create_editor_window ();
+	new_window = gconf_editor_application_create_editor_window (GCONF_EDITOR_WINDOW_NORMAL);
+	gtk_widget_show (new_window);
+}
+
+static void
+gconf_editor_window_new_defaults_window (GtkAction *action, GtkWidget *callback_data)
+{
+	GtkWidget *new_window;
+
+	new_window = gconf_editor_application_create_editor_window (GCONF_EDITOR_WINDOW_DEFAULTS);
+	gtk_widget_show (new_window);
+}
+
+static void
+gconf_editor_window_new_mandatory_window (GtkAction *action, GtkWidget *callback_data)
+{
+	GtkWidget *new_window;
+
+	new_window = gconf_editor_application_create_editor_window (GCONF_EDITOR_WINDOW_MANDATORY);
 	gtk_widget_show (new_window);
 }
 
@@ -212,6 +230,7 @@ gconf_editor_window_about_window (GtkAction *action, GtkWidget *callback_data)
 	GdkPixbuf *pixbuf = NULL;
 	const gchar *authors[] = {
 		"Anders Carlsson <andersca@gnome.org>",
+		"Fernando Herrera <fherrera@onirica.com>",
 		NULL
 	};
 	const gchar *documenters[] = {
@@ -260,7 +279,7 @@ gconf_editor_popup_window_unset_key (GtkAction *action, GtkWidget *callback_data
 			    GCONF_LIST_MODEL_KEY_PATH_COLUMN, &key,
 			    -1);
 
-	gconf_client_unset (gconf_client_get_default (), key, &error);
+	gconf_client_unset (window->client, key, &error);
 	g_free (key);
 
 	if (error != NULL) {
@@ -271,7 +290,8 @@ gconf_editor_popup_window_unset_key (GtkAction *action, GtkWidget *callback_data
 
 static void
 gconf_editor_new_key_response (GtkDialog *editor,
-			       int        response)
+			       int        response,
+			       GConfEditorWindow *window)
 {
 	if (response == GTK_RESPONSE_OK) {
 		GConfValue *value;
@@ -305,7 +325,7 @@ gconf_editor_new_key_response (GtkDialog *editor,
 		/* Create the key */
 		value = gconf_key_editor_get_value (GCONF_KEY_EDITOR (editor));
 
-		gconf_client_set (gconf_client_get_default (), full_path, value, NULL);
+		gconf_client_set (window->client, full_path, value, NULL);
 
 		if(value)
 		  gconf_value_free (value);
@@ -336,34 +356,33 @@ gconf_editor_popup_window_new_key (GtkAction *action, GtkWidget *callback_data)
 	}
 	
 	gconf_key_editor_set_key_path (GCONF_KEY_EDITOR (editor), path);
+	gconf_key_editor_set_writable (GCONF_KEY_EDITOR (editor), TRUE);
 	g_free (path);
 
 	g_signal_connect (editor, "response",
-			  G_CALLBACK (gconf_editor_new_key_response), NULL);
+			  G_CALLBACK (gconf_editor_new_key_response), window);
 
 	gtk_widget_show (editor);
 }
 
 static void
 gconf_editor_edit_key_response (GtkDialog *editor,
-				int        response)
+				int        response,
+				GConfEditorWindow *window)
 {
 	if (response == GTK_RESPONSE_OK) {
 		GConfValue *value;
 		GError     *error = NULL;
 		const char *path;
-		GConfClient *client;
 
 		value = gconf_key_editor_get_value (GCONF_KEY_EDITOR (editor));
 
 		path = gconf_key_editor_get_key_name (GCONF_KEY_EDITOR (editor));
 		g_assert (gconf_valid_key (path, NULL));
 
-		client = gconf_client_get_default ();
 		/* if not writable we weren't allowed to change things anyway */
-		if (gconf_client_key_is_writable (client, path, &error))
-			gconf_client_set (gconf_client_get_default (),
-					  path, value, &error);
+		if (gconf_client_key_is_writable (window->client, path, &error))
+			gconf_client_set (window->client, path, value, &error);
 
 		gconf_value_free (value);
 
@@ -421,12 +440,12 @@ gconf_editor_popup_window_edit_key (GtkAction *action, GtkWidget *callback_data)
 	gconf_key_editor_set_key_name (GCONF_KEY_EDITOR (editor), path);
 
 	gconf_key_editor_set_writable (GCONF_KEY_EDITOR (editor),
-				       gconf_client_key_is_writable (gconf_client_get_default (), path, NULL));
+				       gconf_client_key_is_writable (window->client, path, NULL));
 
 	g_free (path);
 
 	g_signal_connect (editor, "response",
-			  G_CALLBACK (gconf_editor_edit_key_response), NULL);
+			  G_CALLBACK (gconf_editor_edit_key_response), window);
 
 	gtk_widget_show (editor);
 }
@@ -486,11 +505,8 @@ gconf_editor_popup_window_set_as_default (GtkAction *action, GtkWidget *callback
 
   GError *error = NULL;
   GtkTreeIter iter;
-  GConfSchema *schema;
   GConfValue *value;
-  GConfEntry *entry;
   char *path = NULL;
-  const char *schema_path = NULL;
   
   if (defaults_engine == NULL) {
     defaults_engine = gconf_engine_get_for_address (GCONF_DEFAULTS_SOURCE, &error);
@@ -515,47 +531,58 @@ gconf_editor_popup_window_set_as_default (GtkAction *action, GtkWidget *callback
                       GCONF_LIST_MODEL_VALUE_COLUMN, &value,
                       -1);
 
-  entry = gconf_client_get_entry (defaults_client, path, NULL, TRUE, &error);
-  if (error) {
-    gconf_editor_window_popup_error_dialog (window, _("Could not get schema location. Error was:\n%s"), error);
-    return;
-  }
-  g_assert (entry != NULL);
+  gconf_client_set (defaults_client, path, value, NULL);
 
-  schema_path = gconf_entry_get_schema_name (entry);
-  if (schema_path == NULL) {
-    /* This key does not have an existing schema, so lets make one up. */
-    schema_path = g_strdup_printf ("/schemas%s", path);
-  }
-  schema = gconf_client_get_schema (defaults_client, schema_path, &error);
+  gconf_client_suggest_sync (defaults_client, &error);
   if (error) {
-    gconf_editor_window_popup_error_dialog (window, _("Could not get schema. Error was:\n%s"), error);
+    gconf_editor_window_popup_error_dialog (window, _("Could not sync value. Error was:\n%s"), error);
     return;
-  }
-  if (schema == NULL) {
-    /* If we didn't get a schema it doesn't exist yet, so create one now. */
-    schema = gconf_schema_new ();
   }
+}
 
-  gconf_schema_set_default_value (schema, value);
+static void
+gconf_editor_popup_window_set_as_mandatory (GtkAction *action, GtkWidget *callback_data)
+{
+  GConfEditorWindow *gconfwindow = GCONF_EDITOR_WINDOW (callback_data);
+  GtkWindow *window = GTK_WINDOW (callback_data);
+  static GConfEngine *defaults_engine = NULL;
+  static GConfClient *defaults_client = NULL;
 
-  if (!gconf_client_set_schema (defaults_client, schema_path, schema, &error)) {
-    gconf_editor_window_popup_error_dialog (window, _("Could not set schema. Error was:\n%s"), error);
-    return;
+  GError *error = NULL;
+  GtkTreeIter iter;
+  GConfValue *value;
+  char *path = NULL;
+  
+  if (defaults_engine == NULL) {
+    defaults_engine = gconf_engine_get_for_address (GCONF_MANDATORY_SOURCE, &error);
+    if (error) {
+      gconf_editor_window_popup_error_dialog (window, _("Cannot create GConf engine. Error was:\n%s"), error);
+      return;
+    }
+  }
+  if (defaults_client == NULL) {
+    defaults_client = gconf_client_get_for_engine (defaults_engine);
+    if (defaults_client == NULL) {
+      /* TODO: error dialog */
+      g_print ("Could not create GConf client\n");
+      return;
+    }
   }
+	
+  gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (gconfwindow->list_view)),
+                                   NULL, &iter);
+  gtk_tree_model_get (gconfwindow->sorted_list_model, &iter,
+                      GCONF_LIST_MODEL_KEY_PATH_COLUMN, &path,
+                      GCONF_LIST_MODEL_VALUE_COLUMN, &value,
+                      -1);
 
-  /* Docs say not to mix client and engine calls, but Havoc let me */
-  if (!gconf_engine_associate_schema (defaults_engine, path, schema_path, &error)) {
-    gconf_editor_window_popup_error_dialog (window, _("Could not associate schema. Error was:\n%s"), error);
-    return;
-  }
+  gconf_client_set (defaults_client, path, value, NULL);
 
   gconf_client_suggest_sync (defaults_client, &error);
   if (error) {
     gconf_editor_window_popup_error_dialog (window, _("Could not sync value. Error was:\n%s"), error);
     return;
   }
-  /* TODO: free the schema? */
 }
 
 static GtkActionEntry entries[] = {
@@ -564,8 +591,15 @@ static GtkActionEntry entries[] = {
 	{ "BookmarksMenu", NULL, N_("_Bookmarks"), NULL, NULL, NULL },
 	{ "HelpMenu", NULL, N_("_Help"), NULL, NULL, NULL },
 
-	{ "NewWindow", GTK_STOCK_NEW, N_("_New window"), "<control>N", N_("Open a new gconf editor window"), 
+	{ "NewWindow", GTK_STOCK_NEW, N_("New _settings window"), "<control>S", 
+	  N_("Open a new gconf editor window editing current settings"), 
 	  G_CALLBACK (gconf_editor_window_new_window) },
+	{ "NewDefaultsWindow", GTK_STOCK_NEW, N_("New _defaults window"), "<control>D", 
+          N_("Open a new gconf editor window editing system default settings"), 
+	  G_CALLBACK (gconf_editor_window_new_defaults_window) },
+	{ "NewMandatoryWindow", GTK_STOCK_NEW, N_("New _mandatory window"), "<control>M", 
+          N_("Open a new gconf editor window editing system mandatory settings"), 
+	  G_CALLBACK (gconf_editor_window_new_mandatory_window) },
 	{ "CloseWindow", GTK_STOCK_CLOSE, N_("_Close window"), "<control>W", N_("Close this window"), 
 	  G_CALLBACK (gconf_editor_window_close_window) },
 	{ "QuitGConfEditor", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q", N_("Quit the gconf editor"), 
@@ -591,7 +625,9 @@ static GtkActionEntry entries[] = {
 	{ "UnsetKey", GTK_STOCK_DELETE, N_("_Unset key..."), NULL, N_("Unset the selected key"),
 	  G_CALLBACK (gconf_editor_popup_window_unset_key) },
 	{ "DefaultKey", NULL, N_("Set as _default"), NULL, N_("Set the selected key to be the default"),
-	  G_CALLBACK (gconf_editor_popup_window_set_as_default) }	
+	  G_CALLBACK (gconf_editor_popup_window_set_as_default) },
+	{ "MandatoryKey", NULL, N_("Set as _mandatory"), NULL, N_("Set the selected key to the mandatory"),
+	  G_CALLBACK (gconf_editor_popup_window_set_as_mandatory) }	
 };
 
 static const char *ui_description = 
@@ -599,6 +635,8 @@ static const char *ui_description =
 	"	<menubar name='GConfEditorMenu'>"
 	"		<menu action='FileMenu'>"
 	"			<menuitem action='NewWindow'/>"
+	"			<menuitem action='NewDefaultsWindow'/>"
+	"			<menuitem action='NewMandatoryWindow'/>"
 	"			<menuitem action='CloseWindow'/>"
 	"			<separator/>"
 	"			<menuitem action='QuitGConfEditor'/>"
@@ -622,11 +660,15 @@ static const char *ui_description =
 	"		<menuitem action='UnsetKey'/>"
 	"		<separator/>"
 	"		<menuitem action='DefaultKey'/>"	
+	"		<separator/>"
+	"		<menuitem action='MandatoryKey'/>"	
 	"	</popup>"
 	"</ui>";
 
 static gchar *image_menu_items_paths[] = {
 	"/GConfEditorMenu/FileMenu/NewWindow",
+	"/GConfEditorMenu/FileMenu/NewDefaultsWindow",
+	"/GConfEditorMenu/FileMenu/NewMandatoryWindow",
 	"/GConfEditorMenu/FileMenu/CloseWindow",
 	"/GConfEditorMenu/FileMenu/QuitGConfEditor",
 
@@ -738,6 +780,7 @@ gconf_editor_gconf_value_changed (GConfCellRenderer *cell, const gchar *path_str
 	GtkTreeIter iter;
 	GtkTreePath *path;
 	gchar *key;
+	GConfValue *value;
 	
 	path = gtk_tree_path_new_from_string (path_str);
 
@@ -745,11 +788,17 @@ gconf_editor_gconf_value_changed (GConfCellRenderer *cell, const gchar *path_str
 
 	gtk_tree_model_get (window->sorted_list_model, &iter,
 			    GCONF_LIST_MODEL_KEY_PATH_COLUMN, &key,
+			    GCONF_LIST_MODEL_VALUE_COLUMN, &value,
 			    -1);
 
-	gconf_client_set (gconf_client_get_default (), key, new_value, NULL);
+	/* We need to check this because the changed signal could come from an old
+	 * cell in another list_model */ 
+	if (value->type == new_value->type)
+		gconf_client_set (window->client, key, new_value, NULL);
 
+	g_free (value);
 	g_free (key);
+	gtk_tree_path_free (path);
 }
 
 static gboolean
@@ -768,7 +817,7 @@ gconf_editor_check_writable (GConfCellRenderer *cell, const gchar *path_str, GCo
 			    GCONF_LIST_MODEL_KEY_PATH_COLUMN, &key,
 			    -1);
 
-	ret = gconf_client_key_is_writable (gconf_client_get_default (), key, NULL);
+	ret = gconf_client_key_is_writable (window->client, key, NULL);
 
 	g_free (key);
 
@@ -850,7 +899,6 @@ gconf_editor_window_update_list_selection (GtkTreeSelection *selection, GConfEdi
 {
 	GtkTreeIter iter;
 	GConfSchema *schema;
-	GConfClient *client;
 	char *path;
 	
 	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
@@ -870,14 +918,12 @@ gconf_editor_window_update_list_selection (GtkTreeSelection *selection, GConfEdi
 	
 	gtk_label_set_text (GTK_LABEL (window->key_name_label), path);
 
-	client = gconf_client_get_default ();
-
-	if (gconf_client_key_is_writable (client, path, NULL))
+	if (gconf_client_key_is_writable (window->client, path, NULL))
 		gtk_widget_hide (window->non_writable_label);
 	else
 		gtk_widget_show (window->non_writable_label);
 
-	schema = gconf_client_get_schema_for_key (client, path);
+	schema = gconf_client_get_schema_for_key (window->client, path);
 
 	if (schema != NULL && gconf_schema_get_long_desc (schema) != NULL) {
 		char *long_desc;
@@ -910,7 +956,7 @@ gconf_editor_window_update_list_selection (GtkTreeSelection *selection, GConfEdi
 	}
 
 	if (schema == NULL) {
-		GConfValue *value = gconf_client_get (client, path, NULL);
+		GConfValue *value = gconf_client_get (window->client, path, NULL);
 		if (value->type == GCONF_VALUE_SCHEMA) {
 			gtk_widget_hide (window->no_schema_label);
 		}
@@ -1060,6 +1106,17 @@ gconf_editor_window_style_set (GtkWidget *widget, GtkStyle *prev_style, GtkWidge
 				&GTK_WIDGET (widget)->style->bg[GTK_STATE_NORMAL]);
 }
 
+void
+gconf_editor_window_expand_first (GConfEditorWindow *window)
+{
+	GtkTreePath *path;
+
+	path = gtk_tree_path_new_first ();
+	gtk_tree_view_expand_row (GTK_TREE_VIEW (window->tree_view), path, FALSE);
+	gtk_tree_path_free (path);
+}
+
+
 static void
 gconf_editor_window_finalize (GObject *object)
 {
@@ -1078,6 +1135,8 @@ gconf_editor_window_finalize (GObject *object)
 
 	g_object_unref (client);
 
+	/* FIXME: unref client? */
+
         parent_class->finalize (object);
 }
 
@@ -1101,7 +1160,6 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	GtkAccelGroup *accel_group;
 	GtkActionGroup *action_group;
 	GtkWidget *menubar;
-	GtkTreePath *path;
 	GtkWidget *details_frame, *alignment, *table, *label, *text_view, *image;
 	gchar *markup;
 	GError *error;
@@ -1134,6 +1192,19 @@ gconf_editor_window_init (GConfEditorWindow *window)
 
 	window->popup_menu = gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu");
 
+	if (!gconf_client_can_edit_defaults ()) {
+		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/DefaultKey"),
+					  FALSE);
+		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfEditorMenu/FileMenu/NewDefaultsWindow"),
+					  FALSE);
+	}
+	if (!gconf_client_can_edit_mandatory ()) {
+		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/MandatoryKey"),
+					  FALSE);
+		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfEditorMenu/FileMenu/NewMandatoryWindow"),
+					  FALSE);
+	}
+
 	/* Hook up bookmarks */
 	gconf_bookmarks_hook_up_menu (window,
 				      gtk_menu_item_get_submenu (GTK_MENU_ITEM (gtk_ui_manager_get_widget (window->ui_manager,
@@ -1168,10 +1239,6 @@ gconf_editor_window_init (GConfEditorWindow *window)
 			  window);
 	g_object_unref (G_OBJECT (window->tree_model));
 
-	path = gtk_tree_path_new_first ();
-	gtk_tree_view_expand_row (GTK_TREE_VIEW (window->tree_view), path, FALSE);
-	gtk_tree_path_free (path);
-
 	column = gtk_tree_view_column_new ();
 
 	cell = gtk_cell_renderer_pixbuf_new ();
