@@ -66,7 +66,7 @@ main (gint argc, gchar **argv)
 	/* Register our stock icons */
 	gconf_stock_icons_register ();
 
-	window = gconf_editor_application_create_editor_window ();
+	window = gconf_editor_application_create_editor_window (GCONF_EDITOR_WINDOW_NORMAL);
 	gtk_widget_show_now (window);
 
 	if (argc > 1) {
